package tech.nevets.vcardgen;

import org.eclipse.jetty.websocket.api.*;
import org.eclipse.jetty.websocket.api.annotations.*;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;

@WebSocket
public class LiveGenWebSocket {

    private Card card;

    @OnWebSocketConnect
    public void connected(Session session) {}

    @OnWebSocketClose
    public void closed(Session session, int statusCode, String reason) {
        card = null;
    }

    @OnWebSocketError
    public void error(Session session, Throwable throwable) {
        throwable.printStackTrace();
    }

    @OnWebSocketMessage
    public void message(Session session, String message) throws IOException {
        RemoteEndpoint client = session.getRemote();

        System.out.println(message);
        String[] splitMsg = message.split(";");
        String messageType = splitMsg[0];

        if (splitMsg.length == 1) {
            if (messageType.equals("start")) {
                card = new Card();
                client.sendString("success;Card successfully created.");
            }
            return;
        }
        String messageContent = splitMsg[1];

        switch (messageType) {
            case "continue" -> {
                card = Card.CARD_SESSIONS.get(messageContent);
                if (card == null) {
                    card = new Card();
                    client.sendString("notExist;Card with that id does not exist, a new card has been created.");
                } else {
                    client.sendString("success;Card loaded successfully.");
                }
            }
            case "background" -> {
                client.sendString("background");
                client.sendBytes(byteBufferFromImage(card.getBackground()));
            }
            case "location" -> {
                card.setLocation(Location.getLocation(messageContent));
                client.sendString("location");
                client.sendBytes(byteBufferFromImage(card.renderLocationLayer()));
            }
            case "name" -> {
                card.setName(messageContent);
                client.sendString("name");
                client.sendBytes(byteBufferFromImage(card.renderNameLayer()));
            }
            case "title" -> {
                card.setTitle(messageContent);
                client.sendString("title");
                client.sendBytes(byteBufferFromImage(card.renderTitleLayer()));
            }
            case "email" -> {
                card.setEmail(messageContent);
                client.sendString("email");
                client.sendBytes(byteBufferFromImage(card.renderEmailLayer()));
            }
            case "extension" -> {
                card.setExtension(messageContent);
                client.sendString("extension");
                client.sendBytes(byteBufferFromImage(card.renderPhoneNumbersLayer()));
            }
            case "directNumber" -> {
                card.setDirectNumber(messageContent);
                client.sendString("directNumber");
                client.sendBytes(byteBufferFromImage(card.renderPhoneNumbersLayer()));
            }
            case "cellNumber" -> {
                card.setCellNumber(messageContent);
                client.sendString("cellNumber");
                client.sendBytes(byteBufferFromImage(card.renderPhoneNumbersLayer()));
            }
            default -> client.sendString("Unknown Message Type: " + messageType);
        }
    }

    private ByteBuffer byteBufferFromImage(BufferedImage image) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
            ImageIO.write(image, "png", baos);
        } catch (IOException e) {
            e.printStackTrace();
            return ByteBuffer.wrap(("Error writing image: " + e.getMessage()).getBytes(StandardCharsets.UTF_8));
        }
        return ByteBuffer.wrap(baos.toByteArray());
    }
}
